@@ -25,6 +25,7 @@
 #include <sstream>
 #include <iostream>
 #include <algorithm>
+#include <functional>
 
 #define DEBUG_FORMULA_CELL 0
 #if DEBUG_FORMULA_CELL
@@ -305,20 +306,33 @@ std::vector<const formula_token*> formula_cell::get_ref_tokens(
     if (!this_tokens)
         return ret;
 
-    std::for_each(this_tokens->begin(), this_tokens->end(),
-        [&](const formula_tokens_t::value_type& t)
+    std::function<void(const formula_tokens_t::value_type&)> get_refs = [&](const formula_tokens_t::value_type& t)
+    {
+        switch (t->get_opcode())
         {
-            switch (t->get_opcode())
+            case fop_single_ref:
+            case fop_range_ref:
+                ret.push_back(t.get());
+                break;
+            case fop_named_expression:
             {
-                case fop_single_ref:
-                case fop_range_ref:
-                    ret.push_back(t.get());
+                const formula_tokens_t* named_exp =
+                    cxt.get_named_expression(pos.sheet, t->get_name());
+
+                if (!named_exp)
+                    // silently ignore non-existing names.
+                    break;
+
+                // recursive call.
+                std::for_each(named_exp->begin(), named_exp->end(), get_refs);
                 break;
-                default:
-                    ; // ignore the rest.
             }
+            default:
+                ; // ignore the rest.
         }
-    );
+    };
+
+    std::for_each(this_tokens->begin(), this_tokens->end(), get_refs);
 
     return ret;
 }
