@@ -152,7 +152,8 @@ void formula_interpreter::init_tokens()
         if (p->get_opcode() == fop_named_expression)
         {
             // Named expression.  Expand it.
-            const formula_cell* expr = m_context.get_named_expression(p->get_name());
+            const formula_tokens_t* expr = m_context.get_named_expression(
+                m_pos.sheet, p->get_name());
             used_names.insert(p->get_name());
             expand_named_expression(p->get_name(), expr, used_names);
         }
@@ -176,14 +177,18 @@ void get_result_from_cell(const iface::formula_model_access& cxt, const abs_addr
                 return;
 
             res = fcell->get_result_cache();
+            break;
         }
-        break;
+        case celltype_t::boolean:
+            // TODO : treat this as a numeric value for now.  Later we need to
+            // decide whether we need to treat this as a distinct boolean
+            // type.
         case celltype_t::numeric:
             res.set_value(cxt.get_numeric_value(addr));
-        break;
+            break;
         case celltype_t::string:
             res.set_string(cxt.get_string_identifier(addr));
-        break;
+            break;
         case celltype_t::unknown:
         default:
             ;
@@ -228,22 +233,13 @@ void formula_interpreter::pop_result()
 }
 
 void formula_interpreter::expand_named_expression(
-    const string& expr_name, const formula_cell* expr, name_set& used_names)
+    const string& expr_name, const formula_tokens_t* expr, name_set& used_names)
 {
     if (!expr)
-    {
-        ostringstream os;
-        os << "unable to find named expression '" << expr_name << "'";
-        throw invalid_expression(os.str());
-    }
-
-    size_t tokens_id = expr->get_identifier();
-    const formula_tokens_t* expr_tokens = m_context.get_formula_tokens(global_scope, tokens_id);
-    if (!expr_tokens)
-        return;
+        throw formula_error(formula_error_t::name_not_found);
 
     m_tokens.push_back(&paren_open);
-    formula_tokens_t::const_iterator itr = expr_tokens->begin(), itr_end = expr_tokens->end();
+    formula_tokens_t::const_iterator itr = expr->begin(), itr_end = expr->end();
     for (; itr != itr_end; ++itr)
     {
         const formula_token& t = **itr;
@@ -255,7 +251,7 @@ void formula_interpreter::expand_named_expression(
                 // Circular reference detected.
                 throw invalid_expression("circular referencing of named expressions");
             }
-            const formula_cell* expr = m_context.get_named_expression(expr_name);
+            const formula_tokens_t* expr = m_context.get_named_expression(m_pos.sheet, expr_name);
             used_names.insert(expr_name);
             expand_named_expression(expr_name, expr, used_names);
         }
@@ -336,6 +332,10 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
                     val = 0.0;
                     return true;
                 }
+                case celltype_t::boolean:
+                    // TODO : Decide whether we need to treat this as a
+                    // distinct boolean value.  For now, let's treat this as a
+                    // numeric value equivalent.
                 case celltype_t::numeric:
                 {
                     vt = stack_value_t::value;
