@@ -27,6 +27,29 @@ namespace ixion {
 
 namespace {
 
+bool check_address_by_sheet_bounds(const iface::formula_model_access* cxt, const address_t& pos)
+{
+    sheet_size_t ss(row_upper_bound, column_upper_bound);
+
+    if (cxt && pos.sheet >= 0 && size_t(pos.sheet) < cxt->get_sheet_count())
+    {
+        // Make sure the address is within the sheet size.
+        ss = cxt->get_sheet_size(pos.sheet);
+    }
+
+    row_t row_check = pos.row >= 0 ? pos.row : -pos.row;
+
+    if (pos.row != row_unset && row_check >= ss.row)
+        return false;
+
+    col_t col_check = pos.column >= 0 ? pos.column : -pos.column;
+
+    if (pos.column != column_unset && col_check >= ss.column)
+        return false;
+
+    return true;
+}
+
 bool resolve_function(const char* p, size_t n, formula_name_t& ret)
 {
     formula_function_t func_oc = formula_functions::get_function_opcode(p, n);
@@ -598,6 +621,9 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
             if (addr.column)
                 addr.column *= 26;
             addr.column += static_cast<col_t>(c - 'A' + 1);
+
+            if (addr.column > column_upper_bound)
+                return invalid;
         }
         else if (is_digit(c))
         {
@@ -1067,6 +1093,18 @@ public:
         __IXION_DEBUG_OUT__ << "parse address result: " << parse_address_result_names[parse_res] << endl;
 #endif
 
+        if (parse_res != invalid)
+        {
+            // This is a valid A1-style address syntax-wise.
+
+            if (parsed_addr.sheet == invalid_sheet)
+                // sheet name is not found in the model.  Report back as invalid.
+                return ret;
+
+            if (!check_address_by_sheet_bounds(mp_cxt, parsed_addr))
+                parse_res = invalid;
+        }
+
         // prevent for example H to be recognized as column address
         if (parse_res == valid_address && parsed_addr.row != row_unset)
         {
@@ -1224,6 +1262,7 @@ public:
         if (resolve_function(p, n, ret))
             return ret;
 
+        const char* p_end = p + n;
         const char* p_last = p;
         std::advance(p_last, n-1);
 
@@ -1231,6 +1270,19 @@ public:
         address_t parsed_addr(pos.sheet, 0, 0);
 
         parse_address_result parse_res = parse_address_excel_r1c1(mp_cxt, p, p_last, parsed_addr);
+
+        if (parse_res != invalid)
+        {
+            // This is a valid R1C1-style address syntax-wise.
+
+            if (parsed_addr.sheet == invalid_sheet)
+                // sheet name is not found in the model.  Report back as invalid.
+                return ret;
+
+            if (!check_address_by_sheet_bounds(mp_cxt, parsed_addr))
+                parse_res = invalid;
+        }
+
         switch (parse_res)
         {
             case parse_address_result::valid_address:
@@ -1238,10 +1290,12 @@ public:
                 set_cell_reference(ret, parsed_addr);
                 return ret;
             }
-            break;
             case parse_address_result::range_expected:
             {
                 ++p; // skip ':'
+                if (p == p_end)
+                    return ret;
+
                 address_t parsed_addr2(0, 0, 0);
                 parse_address_result parse_res2 = parse_address_excel_r1c1(nullptr, p, p_last, parsed_addr2);
                 if (parse_res2 != parse_address_result::valid_address)
@@ -1254,12 +1308,13 @@ public:
                 set_address(ret.range.first, parsed_addr);
                 set_address(ret.range.last, parsed_addr2);
                 ret.type = formula_name_t::range_reference;
+                return ret;
             }
-            break;
             default:
                 ;
         }
 
+        resolve_function_or_name(p, n, ret);
         return ret;
     }
 
