@@ -87,11 +87,11 @@ void test_name_resolver_excel_a1()
     cout << "test name resolver excel a1" << endl;
 
     model_context cxt;
-    cxt.append_sheet(IXION_ASCII("One"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("Three"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 1024); // name with space
-    cxt.append_sheet(IXION_ASCII("'quote'"), 1048576, 1024); // quoted name
+    cxt.append_sheet(IXION_ASCII("One"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("Three"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 16384); // name with space
+    cxt.append_sheet(IXION_ASCII("'quote'"), 1048576, 16384); // quoted name
     auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
     assert(resolver);
 
@@ -203,13 +203,52 @@ void test_name_resolver_excel_a1()
         formula_name_t res = resolver->resolve(&name_a1[0], name_a1.size(), abs_address_t());
         assert(res.type == name_tests[i].type);
     }
+
+    // Parse address with non-existing sheet name.  It should be flagged invalid.
+
+    res = resolver->resolve(IXION_ASCII("NotExists!A1"), abs_address_t());
+    assert(res.type == formula_name_t::invalid);
+}
+
+void test_name_resolver_named_expression()
+{
+    cout << "Testing the name resolvers for parsing named expressions." << endl;
+
+    model_context cxt;
+    cxt.append_sheet(IXION_ASCII("Sheet"), 1048576, 16384);
+
+    const std::vector<formula_name_resolver_t> resolver_types = {
+        formula_name_resolver_t::excel_a1,
+        formula_name_resolver_t::excel_r1c1,
+        // TODO : add more resolver types.
+    };
+
+    const std::vector<std::string> names = {
+        "MyRange",
+        "MyRange2",
+    };
+
+    for (formula_name_resolver_t rt : resolver_types)
+    {
+        cout << "formula resolver type: " << int(rt) << endl; // TODO : map the enum value to string name.
+
+        auto resolver = formula_name_resolver::get(rt, &cxt);
+        assert(resolver);
+
+        for (const std::string& name : names)
+        {
+            cout << "parsing '" << name << "'..." << endl;
+            formula_name_t res = resolver->resolve(name.data(), name.size(), abs_address_t(0,0,0));
+            assert(res.type == formula_name_t::name_type::named_expression);
+        }
+    }
 }
 
 void test_name_resolver_table_excel_a1()
 {
     cout << "Testing the Excel A1 name resolver for parsing table references." << endl;
     model_context cxt;
-    cxt.append_sheet(IXION_ASCII("Sheet"), 1048576, 1024);
+    cxt.append_sheet(IXION_ASCII("Sheet"), 1048576, 16384);
     string_id_t s_table1 = cxt.append_string(IXION_ASCII("Table1"));
     string_id_t s_table2 = cxt.append_string(IXION_ASCII("Table2"));
     string_id_t s_cat = cxt.append_string(IXION_ASCII("Category"));
@@ -287,10 +326,10 @@ void test_name_resolver_excel_r1c1()
 {
     cout << "test name resolver excel r1c1" << endl;
     model_context cxt;
-    cxt.append_sheet(IXION_ASCII("One"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 1024); // name with space
-    cxt.append_sheet(IXION_ASCII("80's Music"), 1048576, 1024);
+    cxt.append_sheet(IXION_ASCII("One"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 16384); // name with space
+    cxt.append_sheet(IXION_ASCII("80's Music"), 1048576, 16384);
 
     auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_r1c1, &cxt);
     assert(resolver);
@@ -339,7 +378,7 @@ void test_name_resolver_excel_r1c1()
         }
     }
 
-    // These are supposed to be all invalid.
+    // These are supposed to be all invalid or named expression.
     const char* invalid_address[] = {
         "F",
         "RR",
@@ -363,7 +402,7 @@ void test_name_resolver_excel_r1c1()
         const char* p = invalid_address[i];
         string name_r1c1(p);
         formula_name_t res = resolver->resolve(name_r1c1.data(), name_r1c1.size(), abs_address_t());
-        if (res.type != formula_name_t::invalid)
+        if (res.type != formula_name_t::invalid && res.type != formula_name_t::named_expression)
         {
             cerr << "address " << name_r1c1 << " is expected to be invalid." << endl;
             assert(false);
@@ -504,10 +543,10 @@ void test_name_resolver_odff()
     cout << "test name resolver odff" << endl;
 
     model_context cxt;
-    cxt.append_sheet(IXION_ASCII("One"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 1024);
-    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 1024); // name with space
-    cxt.append_sheet(IXION_ASCII("80's Music"), 1048576, 1024);
+    cxt.append_sheet(IXION_ASCII("One"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 16384);
+    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 16384); // name with space
+    cxt.append_sheet(IXION_ASCII("80's Music"), 1048576, 16384);
 
     auto resolver = formula_name_resolver::get(formula_name_resolver_t::odff, &cxt);
     assert(resolver);
@@ -595,10 +634,9 @@ bool check_formula_expression(
 {
     size_t n = strlen(p);
     cout << "testing formula expression '" << p << "'" << endl;
-    formula_tokens_t tokens;
-    parse_formula_string(cxt, abs_address_t(), resolver, p, n, tokens);
-    std::string str;
-    print_formula_tokens(cxt, abs_address_t(), resolver, tokens, str);
+    formula_tokens_t tokens = parse_formula_string(
+        cxt, abs_address_t(), resolver, p, n);
+    std::string str = print_formula_tokens(cxt, abs_address_t(), resolver, tokens);
     int res = strcmp(p, str.c_str());
     if (res)
         cout << "formula expressions differ: '" << p << "' (before) -> '" << str << "' (after)" << endl;
@@ -638,6 +676,7 @@ void test_parse_and_print_expressions()
     };
 
     model_context cxt;
+    cxt.append_sheet(IXION_ASCII("Test"), 1048576, 16384);
     cxt.append_string(IXION_ASCII("Table1"));
     cxt.append_string(IXION_ASCII("Category"));
     cxt.append_string(IXION_ASCII("Value"));
@@ -683,6 +722,7 @@ void test_function_name_resolution()
     };
 
     model_context cxt;
+    cxt.append_sheet(IXION_ASCII("Test"), 1048576, 16384);
     auto resolver = formula_name_resolver::get(ixion::formula_name_resolver_t::excel_a1, &cxt);
     size_t n = IXION_N_ELEMENTS(valid_names);
     for (size_t i = 0; i < n; ++i)
@@ -722,7 +762,7 @@ void test_model_context_storage()
         auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
         assert(resolver);
 
-        cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
+        cxt.append_sheet(IXION_ASCII("test"), 1048576, 16384);
 
         // Test storage of numeric values.
         volatile double val = 0.1;
@@ -751,7 +791,7 @@ void test_model_context_storage()
         auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
         assert(resolver);
 
-        cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
+        cxt.append_sheet(IXION_ASCII("test"), 1048576, 16384);
         string exp = "1";
         cxt.set_formula_cell(abs_address_t(0,0,0), &exp[0], exp.size(), *resolver);
         cxt.set_formula_cell(abs_address_t(0,2,0), &exp[0], exp.size(), *resolver);
@@ -761,7 +801,7 @@ void test_model_context_storage()
     {
         // Test data area.
         model_context cxt;
-        cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
+        cxt.append_sheet(IXION_ASCII("test"), 1048576, 16384);
 
         abs_range_t area = cxt.get_data_range(0);
         assert(!area.valid());
@@ -796,6 +836,46 @@ void test_model_context_storage()
         abs_range_t test = cxt.get_data_range(0);
         assert(test == area);
     }
+
+    {
+        // Fill up the document model and make sure the data range is still
+        // correct.
+        const row_t row_size = 5;
+        const col_t col_size = 4;
+        model_context cxt;
+        cxt.append_sheet(IXION_ASCII("test"), row_size, col_size);
+        for (row_t row = 0; row < row_size; ++row)
+            for (col_t col = 0; col < col_size; ++col)
+                cxt.set_numeric_cell(abs_address_t(0,row,col), 1.0);
+
+        abs_range_t test = cxt.get_data_range(0);
+
+        assert(test.first.sheet == 0);
+        assert(test.first.row == 0);
+        assert(test.first.column == 0);
+        assert(test.last.sheet == 0);
+        assert(test.last.row == row_size-1);
+        assert(test.last.column == col_size-1);
+    }
+
+    {
+        const row_t row_size = 5;
+        const col_t col_size = 4;
+        model_context cxt;
+        cxt.append_sheet(IXION_ASCII("test"), row_size, col_size);
+        cxt.set_numeric_cell(abs_address_t(0,0,0), 1.0);
+        cxt.set_numeric_cell(abs_address_t(0,row_size-1,0), 1.0);
+        cxt.set_numeric_cell(abs_address_t(0,row_size/2,col_size/2), 1.0);
+
+        abs_range_t test = cxt.get_data_range(0);
+
+        assert(test.first.sheet == 0);
+        assert(test.first.row == 0);
+        assert(test.first.column == 0);
+        assert(test.last.sheet == 0);
+        assert(test.last.row == row_size-1);
+        assert(test.last.column == col_size/2);
+    }
 }
 
 void test_volatile_function()
@@ -806,7 +886,7 @@ void test_volatile_function()
     auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
     assert(resolver);
 
-    cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
+    cxt.append_sheet(IXION_ASCII("test"), 1048576, 16384);
 
     dirty_formula_cells_t dirty_cells;
     modified_cells_t dirty_addrs;
@@ -879,6 +959,7 @@ int main()
     test_string_to_double();
     test_string_pool();
     test_name_resolver_excel_a1();
+    test_name_resolver_named_expression();
     test_name_resolver_table_excel_a1();
     test_name_resolver_excel_r1c1();
     test_name_resolver_odff();
