@@ -16,6 +16,7 @@
 #include "ixion/formula.hpp"
 
 #include "workbook.hpp"
+#include "model_types.hpp"
 
 #include <memory>
 #include <sstream>
@@ -133,11 +134,12 @@ bool set_shared_formula_tokens_to_cell(
     return true;
 }
 
-}
+model_context::session_handler_factory dummy_session_handler_factory;
+
+} // anonymous namespace
 
 class model_context_impl
 {
-    typedef std::map<std::string, unique_ptr<formula_cell>> named_expressions_type;
     typedef std::vector<std::string> strings_type;
     typedef std::vector<std::unique_ptr<std::string>> string_pool_type;
     typedef std::unordered_map<mem_str_buf, string_id_t, mem_str_buf::hash> string_map_type;
@@ -155,8 +157,8 @@ public:
         m_parent(parent),
         mp_config(new config),
         mp_cell_listener_tracker(new cell_listener_tracker(parent)),
-        mp_table_handler(NULL),
-        m_session_factory(ixion::make_unique<model_context::session_handler_factory>())
+        mp_table_handler(nullptr),
+        mp_session_factory(&dummy_session_handler_factory)
     {
     }
 
@@ -181,12 +183,12 @@ public:
 
     std::unique_ptr<iface::session_handler> create_session_handler()
     {
-        return m_session_factory->create();
+        return mp_session_factory->create();
     }
 
-    void set_session_handler_factory(std::unique_ptr<model_context::session_handler_factory>&& factory)
+    void set_session_handler_factory(model_context::session_handler_factory* factory)
     {
-        m_session_factory = std::move(factory);
+        mp_session_factory = factory;
     }
 
     iface::table_handler* get_table_handler()
@@ -218,18 +220,23 @@ public:
     celltype_t get_celltype(const abs_address_t& addr) const;
     double get_numeric_value(const abs_address_t& addr) const;
     double get_numeric_value_nowait(const abs_address_t& addr) const;
+    bool get_boolean_value(const abs_address_t& addr) const;
     string_id_t get_string_identifier(const abs_address_t& addr) const;
     string_id_t get_string_identifier_nowait(const abs_address_t& addr) const;
     string_id_t get_string_identifier(const char* p, size_t n) const;
     const formula_cell* get_formula_cell(const abs_address_t& addr) const;
     formula_cell* get_formula_cell(const abs_address_t& addr);
 
-    void set_named_expression(const char* p, size_t n, formula_cell* cell);
-    formula_cell* get_named_expression(const string& name);
-    const formula_cell* get_named_expression(const string& name) const;
-    const string* get_named_expression_name(const formula_cell* expr) const;
+    void set_named_expression(const char* p, size_t n, std::unique_ptr<formula_tokens_t>&& expr);
+    void set_named_expression(sheet_t sheet, const char* p, size_t n, std::unique_ptr<formula_tokens_t>&& expr);
+
+    const formula_tokens_t* get_named_expression(const string& name) const;
+    const formula_tokens_t* get_named_expression(sheet_t sheet, const string& name) const;
+
     sheet_t get_sheet_index(const char* p, size_t n) const;
     std::string get_sheet_name(sheet_t sheet) const;
+    sheet_size_t get_sheet_size(sheet_t sheet) const;
+    size_t get_sheet_count() const;
     sheet_t append_sheet(const char* p, size_t n, row_t row_size, col_t col_size);
 
     string_id_t append_string(const char* p, size_t n);
@@ -271,6 +278,10 @@ public:
 
     dirty_formula_cells_t get_all_formula_cells() const;
 
+    bool empty() const;
+
+    const worksheet* fetch_sheet(sheet_t sheet_index) const;
+
 private:
     model_context& m_parent;
 
@@ -279,9 +290,9 @@ private:
     config* mp_config;
     cell_listener_tracker* mp_cell_listener_tracker;
     iface::table_handler* mp_table_handler;
-    named_expressions_type m_named_expressions;
+    detail::named_expressions_t m_named_expressions;
 
-    std::unique_ptr<model_context::session_handler_factory> m_session_factory;
+    model_context::session_handler_factory* mp_session_factory;
 
     formula_tokens_store_type m_tokens;
     model_context::shared_tokens_type m_shared_tokens;
@@ -291,35 +302,42 @@ private:
     string m_empty_string;
 };
 
-void model_context_impl::set_named_expression(const char* p, size_t n, formula_cell* cell)
+void model_context_impl::set_named_expression(const char* p, size_t n, std::unique_ptr<formula_tokens_t>&& expr)
 {
     string name(p, n);
     m_named_expressions.insert(
-        named_expressions_type::value_type(
-            name, std::unique_ptr<formula_cell>(cell)));
+        detail::named_expressions_t::value_type(std::move(name), std::move(expr)));
 }
 
-formula_cell* model_context_impl::get_named_expression(const string& name)
+void model_context_impl::set_named_expression(
+    sheet_t sheet, const char* p, size_t n, std::unique_ptr<formula_tokens_t>&& expr)
 {
-    named_expressions_type::iterator itr = m_named_expressions.find(name);
-    return itr == m_named_expressions.end() ? NULL : itr->second.get();
+    detail::named_expressions_t& ns = m_sheets.at(sheet).get_named_expressions();
+    string name(p, n);
+    ns.insert(
+        detail::named_expressions_t::value_type(std::move(name), std::move(expr)));
 }
 
-const formula_cell* model_context_impl::get_named_expression(const string& name) const
+const formula_tokens_t* model_context_impl::get_named_expression(const string& name) const
 {
-    named_expressions_type::const_iterator itr = m_named_expressions.find(name);
-    return itr == m_named_expressions.end() ? NULL : itr->second.get();
+    detail::named_expressions_t::const_iterator itr = m_named_expressions.find(name);
+    return itr == m_named_expressions.end() ? nullptr : itr->second.get();
 }
 
-const string* model_context_impl::get_named_expression_name(const formula_cell* expr) const
+const formula_tokens_t* model_context_impl::get_named_expression(sheet_t sheet, const string& name) const
 {
-    named_expressions_type::const_iterator itr = m_named_expressions.begin(), itr_end = m_named_expressions.end();
-    for (; itr != itr_end; ++itr)
+    const worksheet* ws = fetch_sheet(sheet);
+
+    if (ws)
     {
-        if (itr->second.get() == expr)
-            return &itr->first;
+        const detail::named_expressions_t& ns = ws->get_named_expressions();
+        auto it = ns.find(name);
+        if (it != ns.end())
+            return it->second.get();
     }
-    return NULL;
+
+    // Search the global scope if not found in the sheet local scope.
+    return get_named_expression(name);
 }
 
 sheet_t model_context_impl::get_sheet_index(const char* p, size_t n) const
@@ -346,6 +364,16 @@ std::string model_context_impl::get_sheet_name(sheet_t sheet) const
     return m_sheet_names[sheet];
 }
 
+sheet_size_t model_context_impl::get_sheet_size(sheet_t sheet) const
+{
+    return m_sheets.at(sheet).get_sheet_size();
+}
+
+size_t model_context_impl::get_sheet_count() const
+{
+    return m_sheets.size();
+}
+
 sheet_t model_context_impl::append_sheet(
     const char* p, size_t n, row_t row_size, col_t col_size)
 {
@@ -529,8 +557,8 @@ void model_context_impl::set_shared_formula(
         const formula_name_resolver& resolver)
 {
     // Tokenize the formula string and store it.
-    unique_ptr<formula_tokens_t> tokens(new formula_tokens_t);
-    parse_formula_string(m_parent, addr, resolver, p_formula, n_formula, *tokens);
+    unique_ptr<formula_tokens_t> tokens = ixion::make_unique<formula_tokens_t>(
+        parse_formula_string(m_parent, addr, resolver, p_formula, n_formula));
 
     if (si >= m_shared_tokens.size())
         m_shared_tokens.resize(si+1);
@@ -679,16 +707,25 @@ double model_context_impl::count_range(const abs_range_t& range, const values_t&
                 {
                     case element_type_numeric:
                         match = values_type.is_numeric();
-                    break;
+                        break;
+                    case element_type_boolean:
+                        match = values_type.is_boolean();
+                        break;
                     case element_type_string:
                         match = values_type.is_string();
-                    break;
+                        break;
                     case element_type_empty:
                         match = values_type.is_empty();
-                    break;
+                        break;
                     case element_type_formula:
                         ret += count_formula_block(itb, offset, len, values_type);
-                    break;
+                        break;
+                    default:
+                    {
+                        std::ostringstream os;
+                        os << __FUNCTION__ << ": unhandled block type (" << itb->type << ")";
+                        throw general_error(os.str());
+                    }
                 }
 
                 if (match)
@@ -735,6 +772,19 @@ dirty_formula_cells_t model_context_impl::get_all_formula_cells() const
     return cells;
 }
 
+bool model_context_impl::empty() const
+{
+    return m_sheets.empty();
+}
+
+const worksheet* model_context_impl::fetch_sheet(sheet_t sheet_index) const
+{
+    if (sheet_index < 0 || m_sheets.size() <= size_t(sheet_index))
+        return nullptr;
+
+    return &m_sheets[sheet_index];
+}
+
 void model_context_impl::erase_cell(const abs_address_t& addr)
 {
     worksheet& sheet = m_sheets.at(addr.sheet);
@@ -789,8 +839,10 @@ void model_context_impl::set_string_cell(const abs_address_t& addr, string_id_t
 void model_context_impl::set_formula_cell(
     const abs_address_t& addr, const char* p, size_t n, const formula_name_resolver& resolver)
 {
-    unique_ptr<formula_tokens_t> tokens(new formula_tokens_t);
-    parse_formula_string(m_parent, addr, resolver, p, n, *tokens);
+    unique_ptr<formula_tokens_t> tokens =
+        ixion::make_unique<formula_tokens_t>(
+            parse_formula_string(m_parent, addr, resolver, p, n));
+
     unique_ptr<formula_cell> fcell(new formula_cell);
     if (!set_shared_formula_tokens_to_cell(m_parent, addr, *fcell, *tokens))
     {
@@ -871,8 +923,6 @@ abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
             else
                 // Set the first row to 0, and lock it.
                 range.first.row = 0;
-
-            range.last.column = i;
         }
 
         if (range.last.row < (row_size-1))
@@ -902,10 +952,11 @@ abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
             else
                 // Last block is not empty.
                 range.last.row = row_size - 1;
-
-            range.last.column = i;
         }
 
+        // Check if the column contains at least one non-empty cell.
+        if (col.block_size() > 1 || !col.is_empty(0))
+            range.last.column = i;
     }
 
     if (range.last.column < 0)
@@ -930,12 +981,18 @@ celltype_t model_context_impl::get_celltype(const abs_address_t& addr) const
             return celltype_t::empty;
         case mdds::mtv::element_type_numeric:
             return celltype_t::numeric;
+        case mdds::mtv::element_type_boolean:
+            return celltype_t::boolean;
         case mdds::mtv::element_type_ulong:
             return celltype_t::string;
         case element_type_formula:
             return celltype_t::formula;
         default:
-            throw general_error("unknown cell type");
+        {
+            std::ostringstream os;
+            os << "ixion::model_context_impl::get_celltype: unknown cell type (" << gmcell_type << ")";
+            throw general_error(os.str());
+        }
     }
 
     return celltype_t::unknown;
@@ -948,6 +1005,8 @@ double model_context_impl::get_numeric_value(const abs_address_t& addr) const
     {
         case mdds::mtv::element_type_numeric:
             return col_store.get<double>(addr.row);
+        case mdds::mtv::element_type_boolean:
+            return col_store.get<bool>(addr.row);
         case element_type_formula:
         {
             const formula_cell* p = col_store.get<formula_cell*>(addr.row);
@@ -967,6 +1026,8 @@ double model_context_impl::get_numeric_value_nowait(const abs_address_t& addr) c
     {
         case element_type_numeric:
             return col_store.get<double>(addr.row);
+        case mdds::mtv::element_type_boolean:
+            return col_store.get<bool>(addr.row);
         case element_type_formula:
         {
             const formula_cell* p = col_store.get<formula_cell*>(addr.row);
@@ -979,6 +1040,27 @@ double model_context_impl::get_numeric_value_nowait(const abs_address_t& addr) c
     return 0.0;
 }
 
+bool model_context_impl::get_boolean_value(const abs_address_t& addr) const
+{
+    const column_store_t& col_store = m_sheets.at(addr.sheet).at(addr.column);
+    switch (col_store.get_type(addr.row))
+    {
+        case mdds::mtv::element_type_numeric:
+            return col_store.get<double>(addr.row) != 0.0 ? true : false;
+        case mdds::mtv::element_type_boolean:
+            return col_store.get<bool>(addr.row);
+        case element_type_formula:
+        {
+            const formula_cell* p = col_store.get<formula_cell*>(addr.row);
+            return p->get_value() != 0.0 ? true : false;
+        }
+        break;
+        default:
+            ;
+    }
+    return false;
+}
+
 string_id_t model_context_impl::get_string_identifier(const abs_address_t& addr) const
 {
     const column_store_t& col_store = m_sheets.at(addr.sheet).at(addr.column);
@@ -1136,6 +1218,11 @@ double model_context::get_numeric_value(const abs_address_t& addr) const
     return mp_impl->get_numeric_value(addr);
 }
 
+bool model_context::get_boolean_value(const abs_address_t& addr) const
+{
+    return mp_impl->get_boolean_value(addr);
+}
+
 double model_context::get_numeric_value_nowait(const abs_address_t& addr) const
 {
     return mp_impl->get_numeric_value_nowait(addr);
@@ -1286,24 +1373,30 @@ std::string model_context::get_sheet_name(sheet_t sheet) const
     return mp_impl->get_sheet_name(sheet);
 }
 
-void model_context::set_named_expression(const char* p, size_t n, formula_cell* cell)
+sheet_size_t model_context::get_sheet_size(sheet_t sheet) const
 {
-    mp_impl->set_named_expression(p, n, cell);
+    return mp_impl->get_sheet_size(sheet);
 }
 
-formula_cell* model_context::get_named_expression(const string& name)
+size_t model_context::get_sheet_count() const
 {
-    return mp_impl->get_named_expression(name);
+    return mp_impl->get_sheet_count();
 }
 
-const formula_cell* model_context::get_named_expression(const string& name) const
+void model_context::set_named_expression(const char* p, size_t n, std::unique_ptr<formula_tokens_t>&& expr)
 {
-    return mp_impl->get_named_expression(name);
+    mp_impl->set_named_expression(p, n, std::move(expr));
 }
 
-const string* model_context::get_named_expression_name(const formula_cell* expr) const
+void model_context::set_named_expression(
+    sheet_t sheet, const char* p, size_t n, std::unique_ptr<formula_tokens_t>&& expr)
 {
-    return mp_impl->get_named_expression_name(expr);
+    mp_impl->set_named_expression(sheet, p, n, std::move(expr));
+}
+
+const formula_tokens_t* model_context::get_named_expression(sheet_t sheet, const string& name) const
+{
+    return mp_impl->get_named_expression(sheet, name);
 }
 
 sheet_t model_context::append_sheet(const char* p, size_t n, row_t row_size, col_t col_size)
@@ -1311,9 +1404,9 @@ sheet_t model_context::append_sheet(const char* p, size_t n, row_t row_size, col
     return mp_impl->append_sheet(p, n, row_size, col_size);
 }
 
-void model_context::set_session_handler_factory(std::unique_ptr<session_handler_factory>&& factory)
+void model_context::set_session_handler_factory(session_handler_factory* factory)
 {
-    mp_impl->set_session_handler_factory(std::move(factory));
+    mp_impl->set_session_handler_factory(factory);
 }
 
 void model_context::set_table_handler(iface::table_handler* handler)
@@ -1346,5 +1439,10 @@ dirty_formula_cells_t model_context::get_all_formula_cells() const
     return mp_impl->get_all_formula_cells();
 }
 
+bool model_context::empty() const
+{
+    return mp_impl->empty();
+}
+
 }
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
